module spimem_tb;

parameter ADDRW = 2;

reg sck, copi, cs;
reg [7:0] rdata;
wire cipo, we, re;
wire [ADDRW * 8 - 1:0] addr;
wire [7:0] wdata;
integer nbit;

localparam per = 20; // ns

spimem #(.ADDRW(ADDRW), .LINEW(16)) DUT (
	.sck(sck),
	.copi(copi),
	.cipo(cipo),
	.cs(cs),

	.addr(addr),
	.wdata(wdata),
	.we(we),
	.rdata(rdata),
	.re(re),
	.rst(1'b0)
);

task send_bit;
	input b;
	begin
		copi = b; #per; sck = 1; #per; sck = 0; #per;
	end
endtask

task send_byte;
	input [7:0] b;
	integer i;
	begin
		for (i = 0; i < 8; i = i + 1) begin
			nbit <= i;
			send_bit(b[7 - i]);
		end
	end
endtask

initial
begin
	$dumpfile("spimem_stride_tb.vcd");
	$dumpvars();

	cs = 0;
	copi = 0;
	sck = 0;
	rdata = 'h42;

	cs = 1; #per;
	// addr
	send_byte('h00);
	send_byte('h01);
	// stride
	send_byte('h00);
	send_byte('h02);
	// mode = read
	send_byte('h00);
	// data
	send_byte('h00);
	send_byte('h00);
	send_byte('h00);
	send_byte('h00);
	cs = 0; #per;

	#per; #per;

	cs = 1; #per;
	// addr
	send_byte('h00);
	send_byte('h00);
	// stride
	send_byte('h00);
	send_byte('h00);
	// mode = write
	send_byte('h80);
	// data
	send_byte('h13);
	send_byte('h37);
	send_byte('h13);
	send_byte('h37);
	cs = 0; #per;

end

endmodule
